"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ExecFetcher = void 0;

var _core = require("@yarnpkg/core");

var _fslib = require("@yarnpkg/fslib");

var _tmp = require("tmp");

var _constants = require("./constants");

class ExecFetcher {
  supports(locator, opts) {
    if (!locator.reference.startsWith(_constants.PROTOCOL)) return false;
    return true;
  }

  getLocalPath(locator, opts) {
    const {
      parentLocator,
      path
    } = _core.structUtils.parseFileStyleRange(locator.reference, {
      protocol: _constants.PROTOCOL
    });

    if (_fslib.ppath.isAbsolute(path)) return path;
    const parentLocalPath = opts.fetcher.getLocalPath(parentLocator, opts);
    if (parentLocalPath === null) return null;
    return _fslib.ppath.resolve(parentLocalPath, path);
  }

  async fetch(locator, opts) {
    const expectedChecksum = opts.checksums.get(locator.locatorHash) || null;
    const [packageFs, releaseFs, checksum] = await opts.cache.fetchPackageFromCache(locator, expectedChecksum, async () => {
      opts.report.reportInfoOnce(_core.MessageName.FETCH_NOT_CACHED, `${_core.structUtils.prettyLocator(opts.project.configuration, locator)} can't be found in the cache and will be fetched from the disk`);
      return await this.fetchFromDisk(locator, opts);
    });
    return {
      packageFs,
      releaseFs,
      prefixPath: _core.structUtils.getIdentVendorPath(locator),
      localPath: this.getLocalPath(locator, opts),
      checksum
    };
  }

  async fetchFromDisk(locator, opts) {
    const {
      parentLocator,
      path
    } = _core.structUtils.parseFileStyleRange(locator.reference, {
      protocol: _constants.PROTOCOL
    }); // If the file target is an absolute path we can directly access it via its
    // location on the disk. Otherwise we must go through the package fs.


    const parentFetch = _fslib.ppath.isAbsolute(path) ? {
      packageFs: new _fslib.NodeFS(),
      prefixPath: _fslib.PortablePath.root,
      localPath: _fslib.PortablePath.root
    } : await opts.fetcher.fetch(parentLocator, opts); // If the package fs publicized its "original location" (for example like
    // in the case of "file:" packages), we use it to derive the real location.

    const effectiveParentFetch = parentFetch.localPath ? {
      packageFs: new _fslib.NodeFS(),
      prefixPath: parentFetch.localPath
    } : parentFetch; // Discard the parent fs unless we really need it to access the files

    if (parentFetch !== effectiveParentFetch && parentFetch.releaseFs) parentFetch.releaseFs();
    const generatorFs = effectiveParentFetch.packageFs;

    const generatorPath = _fslib.ppath.resolve(_fslib.ppath.resolve(generatorFs.getRealPath(), effectiveParentFetch.prefixPath), path); // Execute the specified script in the temporary directory


    const cwd = await this.generatePackage(locator, generatorPath, opts); // Make sure the script generated the package

    if (!_fslib.xfs.existsSync(_fslib.ppath.join(cwd, (0, _fslib.toFilename)(`build`)))) throw new Error(`The script should have generated a build directory`);
    return await _core.tgzUtils.makeArchiveFromDirectory(_fslib.ppath.join(cwd, (0, _fslib.toFilename)(`build`)), {
      prefixPath: _core.structUtils.getIdentVendorPath(locator)
    });
  }

  async generatePackage(locator, generatorPath, opts) {
    const cwd = _fslib.npath.toPortablePath((0, _tmp.dirSync)().name);

    const env = await _core.scriptUtils.makeScriptEnv({
      project: opts.project
    });

    const logFile = _fslib.npath.toPortablePath((0, _tmp.tmpNameSync)({
      prefix: `buildfile-`,
      postfix: `.log`
    }));

    const stdin = null;

    const stdout = _fslib.xfs.createWriteStream(logFile);

    const stderr = stdout;
    stdout.write(`# This file contains the result of Yarn generating a package (${_core.structUtils.stringifyLocator(locator)})\n`);
    stdout.write(`\n`);
    const {
      code
    } = await _core.execUtils.pipevp(process.execPath, [_fslib.npath.fromPortablePath(generatorPath), _core.structUtils.stringifyIdent(locator)], {
      cwd,
      env,
      stdin,
      stdout,
      stderr
    });
    if (code !== 0) throw new Error(`Package generation failed (exit code ${code}, logs can be found here: ${logFile})`);
    return cwd;
  }

}

exports.ExecFetcher = ExecFetcher;